{% extends "course_flow/base.html" %}

{% load static i18n %}

{% block title %}{% trans "My Library" %} - CourseFlow{% endblock %}

{% block scripts %}
   {{ block.super }}
 <script src="{% static 'course_flow/js/templates/library.js' %}"></script>
{% endblock %}
