$.ajaxSetup({
  beforeSend: function (xhr, settings) {
    if (!csrfSafeMethod(settings.type) && !this.crossDomain) {
      xhr.setRequestHeader('X-CSRFToken', getCsrfToken())
    }
  }
})

function csrfSafeMethod(method) {
  return /^(GET|HEAD|OPTIONS|TRACE)$/.test(method)
}

export function getCsrfToken() {
  return document
    .getElementsByName('csrfmiddlewaretoken')[0]
    .getAttribute('value')
}
